#!/usr/bin/python3
import wave
import struct

def physical_layer(filename):
    tape = wave.open(filename, 'rb')
    freq = tape.getframerate()

    if tape.getsampwidth()!=2:
        raise Exception('Sorry, sample width is weird: %d' % (
            tape.getsampwidth(),))

    low_byte = False
    current = None
    prev = 0
    wavelength = 0
    in_2400 = False

    samples = tape.readframes(tape.getnframes())

    print(tape.getnframes())
    for i in range(tape.getnframes()):

        sample = struct.unpack('<h', samples[i*2:i*2+2])[0]

        if sample>0 and prev<=0:
            # Rising edge of the waveform. How wide is it?
            if wavelength < 27:
                # Short wavelength = high frequency
                if in_2400:
                    in_2400 = False
                else:
                    in_2400 = True
                    yield 1
            else:
                # Long wavelength = low frequency
                in_2400 = False
                yield 0

            wavelength = 0
        else:
            wavelength += 1

        prev = sample

def data_layer(physical):

    seq = 0
    b = 0
    tone_length = 0

    for bit in physical:

        if seq==0:
            if bit==0:
                seq += 1
                b = 0
            else:
                tone_length += 1

            if tone_length==1000:
                yield None

        elif seq>0 and seq<9:
            b = b>>1 | (bit<<7)
            seq += 1
        elif seq==9:
            if bit==1:
                seq += 1
            else:
                seq = 0
                tone_length = 0
        else:
            if bit==1:
                yield chr(b & 0x7F)
            b = 0
            seq = 0
            tone_length = 0

def main():

    for x in data_layer(
            physical_layer('side2.wav')):

        if x is None:
            print()
            print()
        elif x=='\002':
            pass # STX - begin program
        if x=='\r':
            print(flush=True)
        else:
            print(x, end='', flush=True)

if __name__=='__main__':
    main()
